@@ -102,7 +102,7 @@ impl IsolationMap {
 /// SOCKS handshake and relay the connection over the Tor network.
 ///
 /// Uses `isolation_map` to decide which circuits circuits this connection
-/// may use.  Requires that `isolation_info` is a pair listeng the listener
+/// may use.  Requires that `isolation_info` is a pair listing the listener
 /// id and the source address for the socks request.
 async fn handle_socks_conn<R, S>(
     runtime: R,
@@ -147,10 +147,7 @@ where
             n_read -= action.drain;
         }
         if !action.reply.is_empty() {
-            socks_w
-                .write(&action.reply[..])
-                .await
-                .context("Error while writing reply to SOCKS handshake")?;
+            write_all_and_flush(&mut socks_w, &action.reply[..]).await?;
         }
         if action.finished {
             break handshake.into_request();
@@ -190,7 +187,9 @@ where
         SocksCmd::CONNECT => {
             // The SOCKS request wants us to connect to a given address.
             // So, launch a connection over Tor.
-            let tor_stream = tor_client.connect((addr.clone(), port), Some(prefs)).await;
+            let tor_stream = tor_client
+                .connect_with_prefs((addr.clone(), port), prefs)
+                .await;
             let tor_stream = match tor_stream {
                 Ok(s) => s,
                 // In the case of a stream timeout, send the right SOCKS reply.
@@ -198,31 +197,24 @@ where
                     // The connect attempt has failed.  We need to
                     // send an error.  See what kind it is.
                     //
-                    match e {
+                    let reply = match e {
                         arti_client::Error::Timeout => {
-                            let reply =
-                                request.reply(tor_socksproto::SocksStatus::TTL_EXPIRED, None);
-                            socks_w
-                                .write(&reply[..])
-                                .await
-                                .context("Couldn't write SOCKS reply")?;
-                            return Err(anyhow!(e));
+                            request.reply(tor_socksproto::SocksStatus::TTL_EXPIRED, None)
                         }
-                        _ => return Err(anyhow!(e)),
-                    }
+                        _ => request.reply(tor_socksproto::SocksStatus::GENERAL_FAILURE, None),
+                    };
+                    write_all_and_close(&mut socks_w, &reply[..]).await?;
+                    return Err(anyhow!(e));
                 }
             };
             // Okay, great! We have a connection over the Tor network.
             info!("Got a stream for {}:{}", addr, port);
-            // TODO: XXXX-A1 Should send a SOCKS reply if something fails.
+            // TODO: Should send a SOCKS reply if something fails. See #258.
 
             // Send back a SOCKS response, telling the client that it
             // successfully connected.
             let reply = request.reply(tor_socksproto::SocksStatus::SUCCEEDED, None);
-            socks_w
-                .write(&reply[..])
-                .await
-                .context("Couldn't write SOCKS reply")?;
+            write_all_and_flush(&mut socks_w, &reply[..]).await?;
 
             let (tor_r, tor_w) = tor_stream.split();
 
@@ -234,16 +226,13 @@ where
         SocksCmd::RESOLVE => {
             // We've been asked to perform a regular hostname lookup.
             // (This is a tor-specific SOCKS extension.)
-            let addrs = tor_client.resolve(&addr, Some(prefs)).await?;
+            let addrs = tor_client.resolve_with_prefs(&addr, prefs).await?;
             if let Some(addr) = addrs.first() {
                 let reply = request.reply(
                     tor_socksproto::SocksStatus::SUCCEEDED,
                     Some(&SocksAddr::Ip(*addr)),
                 );
-                socks_w
-                    .write(&reply[..])
-                    .await
-                    .context("Couldn't write SOCKS reply")?;
+                write_all_and_flush(&mut socks_w, &reply[..]).await?;
             }
         }
         SocksCmd::RESOLVE_PTR => {
@@ -254,37 +243,63 @@ where
                 Err(e) => {
                     let reply =
                         request.reply(tor_socksproto::SocksStatus::ADDRTYPE_NOT_SUPPORTED, None);
-                    socks_w
-                        .write(&reply[..])
-                        .await
-                        .context("Couldn't write SOCKS reply")?;
+                    write_all_and_close(&mut socks_w, &reply[..]).await?;
                     return Err(anyhow!(e));
                 }
             };
-            let hosts = tor_client.resolve_ptr(addr, Some(prefs)).await?;
+            let hosts = tor_client.resolve_ptr_with_prefs(addr, prefs).await?;
             if let Some(host) = hosts.into_iter().next() {
                 let reply = request.reply(
                     tor_socksproto::SocksStatus::SUCCEEDED,
                     Some(&SocksAddr::Hostname(host.try_into()?)),
                 );
-                socks_w
-                    .write(&reply[..])
-                    .await
-                    .context("Couldn't write SOCKS reply")?;
+                write_all_and_flush(&mut socks_w, &reply[..]).await?;
             }
         }
         _ => {
             // We don't support this SOCKS command.
             warn!("Dropping request; {:?} is unsupported", request.command());
+            let reply = request.reply(tor_socksproto::SocksStatus::COMMAND_NOT_SUPPORTED, None);
+            write_all_and_close(&mut socks_w, &reply[..]).await?;
         }
     };
 
-    // TODO: XXXX-A1 we should close the TCP stream if either task fails. Do we?
-    // TODO: XXXX-A1 should report the errors.
+    // TODO: we should close the TCP stream if either task fails. Do we?
+    // See #211 and #190.
 
     Ok(())
 }
 
+/// write_all the data to the writer & flush the writer if write_all is successful.
+async fn write_all_and_flush<W>(writer: &mut W, buf: &[u8]) -> Result<()>
+where
+    W: AsyncWrite + Unpin,
+{
+    writer
+        .write_all(buf)
+        .await
+        .context("Error while writing SOCKS reply")?;
+    writer
+        .flush()
+        .await
+        .context("Error while flushing SOCKS stream")
+}
+
+/// write_all the data to the writer & close the writer if write_all is successful.
+async fn write_all_and_close<W>(writer: &mut W, buf: &[u8]) -> Result<()>
+where
+    W: AsyncWrite + Unpin,
+{
+    writer
+        .write_all(buf)
+        .await
+        .context("Error while writing SOCKS reply")?;
+    writer
+        .close()
+        .await
+        .context("Error while closing SOCKS stream")
+}
+
 /// Copy all the data from `reader` into `writer` until we encounter an EOF or
 /// an error.
 ///
@@ -395,7 +410,7 @@ pub(crate) async fn run_socks_proxy<R: Runtime>(
     // We weren't able to bind any ports: There's nothing to do.
     if listeners.is_empty() {
         error!("Couldn't open any listeners.");
-        return Ok(()); // XXXX should return an error.
+        return Err(anyhow!("Couldn't open listeners"));
     }
 
     // Create a stream of (incoming socket, listener_id) pairs, selected
